#!/usr/bin/perl

use strict;
use CGI;
use CGI::Session;

require 'config.pl';
require 'lib/cgilib.pl';
require 'lib/session.pl';
require 'lib/user.pl';
require 'lib/question.pl';

my $cgi = CGI->new;
my $session_id;
my $session;

($session_id, $session) = session::check($cgi);

my $uid = $session->param('uid');
my $name = user::get_name($uid);
my $lines;

if (!-d "./result") {
	mkdir("./result");
}

$lines = make_table();
open(FILE, "> ./result/progress.txt");
print FILE "$lines";
close(FILE);

$lines = make_plot();
open(FILE, "> ./result/progress.gp");
print FILE "$lines";
close(FILE);

cgilib::header($cgi, "Progress");
cgilib::menu($session_id, $name);

open(IMAGEFILE, "cat ./result/progress.gp | $config::gnuplot | $config::convert - png:- | $config::base64 |");

print '<center>';
print "<img src=\"data:image/png;base64,";
while (<IMAGEFILE>) {
	s/\n$//;
	print "$_";
}
print "\">";
print '</center>';

close(IMAGEFILE);

cgilib::footer($cgi);

sub make_table {
	my $lines;
	my @uid_list;
	my @qid_list;
	my $uid;
	my $qid;
	my %time_list;
	my $tdiv;
	my $t;
	my $tmin;
	my $tp;
	my $score;

	@uid_list = user::list();

	$tdiv = 60;

	$t = time();
	$t = int($t / $tdiv) * $tdiv;
	$time_list{$t} = 1;
	$time_list{$t + $tdiv} = 1;

	foreach $uid (@uid_list) {
		@qid_list = question::clear_list($uid);
		foreach $qid (@qid_list) {
			$t = question::get_time($uid, $qid);
			$t = int($t / $tdiv) * $tdiv;
			$time_list{$t} = 1;
			$time_list{$t + $tdiv} = 1;
		}
	}

	foreach $t (sort { $a <=> $b } keys(%time_list)) {
		if (!defined($tmin)) {
			$tmin = $t;
		}
		$tp = int(($t - $tmin) / 60);
		$lines .= "$tp";
		foreach $uid (@uid_list) {
			$score = question::total_score($uid, $t);
			$lines .= "\t$score";
		}
		$lines .=  "\n";
	}

	return $lines;
}

sub make_plot {
	my $lines;
	my @uid_list;
	my $uid;
	my $name;
	my $n;
	my $column;

	@uid_list = user::list();

	$lines .= "set terminal pbm color\n";
	$lines .= "set output\n";
	$lines .= "set xlabel 'Time'\n";
	$lines .= "set ylabel 'Score'\n";
	$lines .= "set key top left\n";
	$n = 0;
	foreach $uid (@uid_list) {
		if ($n == 0) {
			$lines .= "plot ";
		} else {
			$lines .= ", ";
		}
		$name = user::get_name($uid);
		$column = $n + 2;
		$lines .= "'./result/progress.txt' using 1:$column ";
		$lines .= "title '$name' w l lw 5";
		$n++;
	}
	$lines .= "\n";

	return $lines;
}
